# SPDX-FileCopyrightText: Copyright (c) 2019-2025 Aibolit
# SPDX-License-Identifier: MIT


class LocalMethodsCalls:
    """
        Measure the number of local methods called by the class.
        @todo #183:30min Implement local method called metric
         In the methods in the class an local method (method that is defined in this class)
         can be called. So number of local methods calls by all the methods in the
         class is suggested as a metric. Implement this metric and then
         enable tests in test_local_methods_calls.py
    """

    def __init__(self):
        pass

    def value(self, filename: str):
        return 0
