# SPDX-FileCopyrightText: Copyright (c) 2019-2025 Aibolit
# SPDX-License-Identifier: MIT

"""Type declarations module for aibolit.

This module provides type definitions used throughout the aibolit codebase.
"""

LineNumber = int
