# SPDX-FileCopyrightText: Copyright (c) 2019-2025 Aibolit
# SPDX-License-Identifier: MIT

"""
Aibolit: Static Analyzer with Machine Learning in Mind
"""

__version__ = '1.3.0'
__author__ = 'Yegor Bugayenko'
__licence__ = 'MIT'
